/*
 * Copyright 2008-2009 the T2 Project ant the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.t2framework.lucy.tx;

import javax.transaction.Status;
import javax.transaction.SystemException;
import javax.transaction.Transaction;
import javax.transaction.TransactionManager;
import javax.transaction.UserTransaction;

import org.t2framework.commons.util.Logger;

/**
 * 
 * {@.en A processor for JTA transaction.}
 * 
 * <br />
 * 
 * {@.ja }
 * 
 * @author shot
 * 
 */
public class JTATransactionProcesser implements TransactionProcesser,
		TransactionWrapper {

	private static final Logger LOG = Logger
			.getLogger(JTATransactionProcesser.class);

	protected TransactionManager txManager;

	protected UserTransaction userTx;

	public JTATransactionProcesser() {
	}

	@Override
	public <R> R required(Worker<R> worker) throws Throwable {
		boolean began = begin();
		try {
			return worker.work(this);
		} finally {
			if (began) {
				end();
			}
		}
	}

	@Override
	public <R> R requiresNew(Worker<R> worker) throws Throwable {
		Transaction tx = suspend();
		try {
			begin();
			try {
				return worker.work(this);
			} finally {
				end();
			}
		} finally {
			resume(tx);
		}
	}

	@Override
	public <R> R mandatory(Worker<R> worker) throws Throwable {
		if (hasTransaction() == false) {
			throw new IllegalStateException();
		}
		return worker.work(this);
	}

	@Override
	public <R> R never(Worker<R> worker) throws Throwable {
		if (hasTransaction()) {
			throw new IllegalStateException();
		}
		return worker.work(this);
	}

	@Override
	public <R> R notSupported(Worker<R> worker) throws Throwable {
		Transaction tx = suspend();
		try {
			return worker.work(this);
		} finally {
			resume(tx);
		}
	}

	public TransactionManager getTxManager() {
		return txManager;
	}

	public void setTxManager(TransactionManager txManager) {
		this.txManager = txManager;
	}

	public UserTransaction getUserTx() {
		return userTx;
	}

	public void setUserTx(UserTransaction userTx) {
		this.userTx = userTx;
	}

	@Override
	public boolean begin() throws Exception {
		if (hasTransaction() == false) {
			this.userTx.begin();
			return true;
		}
		return false;
	}

	@Override
	public boolean hasTransaction() throws Exception {
		int i = this.userTx.getStatus();
		return i != Status.STATUS_NO_TRANSACTION && i != Status.STATUS_UNKNOWN;
	}

	@Override
	public void resume(Transaction tx) throws Exception {
		this.txManager.resume(tx);
	}

	@Override
	public void setRollbackOnly() {
		try {
			if (this.userTx.getStatus() == Status.STATUS_ACTIVE) {
				this.userTx.setRollbackOnly();
			}
		} catch (SystemException e) {
			LOG.error(e.getLocalizedMessage(), new Object[] { e });
		}
	}

	@Override
	public Transaction suspend() throws Exception {
		return hasTransaction() ? this.txManager.suspend() : null;
	}

	@Override
	public void end() throws Exception {
		if (this.userTx.getStatus() == Status.STATUS_ACTIVE) {
			this.userTx.commit();
		} else {
			this.userTx.rollback();
		}
	}
}
